#
# aplicação que mostra os limites de confiança para uma distribuição normal 
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
resolution <- 0.1
tolerance <- 0.00005

server <- function(input, output) {
 
  # tamanho da amostra   
  sampleSize <- reactive ({
    input$sampleSize
  })
  
  # Média da amostra   
  sampleMean <- reactive ({
    input$sampleMean
  })

  # limite inferior sugerido do intervalo de confiança  
  infLimit <- reactive ({
    input$infLimit
  })
  
  # limite superior sugerido do intervalo de confiança  
  supLimit <- reactive ({
    input$supLimit
  })
  
  # metade do valor de alfa   
  half.alfa <- reactive ({
      (1 - input$conflevel/100)/2
  })
  
  # Erro padrão da média amostral   
  sdError <- reactive ({
    input$dpNormal / sqrt(sampleSize())
  })

  #   valor mínimo do eixo x
  min <- reactive ({
    sampleMean() - 5 * sdError()
  })
  
  #   valor máximo do eixo x
  max <- reactive ({
    sampleMean() + 5 * sdError()
  })

  output$superiorLimit <- renderPlot({
    par(cex=1.5)
    # traça a distribuição da média amostral com erro padrão calculado em função do
    # tamanho da amostra e do desvio padrão da distribuição na população
    curve(dnorm(x,supLimit(),sdError()),xlim=c(min(),max()), main="Limite Superior", xlab = "", ylab = "", yaxt='n', xaxt='n') 
    mtext(paste("N(",sprintf("%.1f", supLimit()),", ",sprintf("%.1f", sdError()*sdError()),")", sep=""), cex = 1.5)
    axis(1, at=c(sampleMean()), lwd.ticks = 2)
    # especifica as coordenadas da área correspondente à cauda inerior (abaixo da média amostral)
    # para o limite superior especificado pelo usuário
    cord.z <- c(min(),seq(min(), sampleMean(), resolution),sampleMean(), sampleMean())
    cord.w <- c(0,dnorm(seq(min(), sampleMean(), resolution), supLimit(), sdError()), dnorm(sampleMean(), supLimit(), sdError()), 0)
    # calcula a área da cauda inferior, abaixo da média da amostra,
    # da distribuição da média amostral quando a média da distribuição é 
    # igual ao limite superior
    p <- pnorm(sampleMean(), supLimit(), sdError(), lower.tail = TRUE)
    # se a área é igual a alfa/2, então traça uma linha vertical azul
    # demarcando o limite superior do intervalo de confiança, faz uma marca no eixo x
    # e desenha a área em verde
    # caso contrário, só desenha a área em vermelho
    color <- if (abs(p - half.alfa()) < tolerance) {
                abline(v=supLimit(), lwd = "2", col = "blue")
                par(col.axis = "blue")
                axis(1, at=c(supLimit()), lwd.ticks = 2)
                rgb(0, 1, 0,0.5)
            } else {
                 rgb(1, 0, 0,0.5)
            }
    polygon(cord.z,cord.w,col=color)
    limits <- par("usr")
    ypos <- 3* par("usr")[4]/4 
    # escreve o valor da área da cauda inferior
    text(min(),  ypos, paste("alfa/2 = ", sprintf("%.4f", p)), pos = 4)
    })

  output$inferiorLimit <- renderPlot({
    par(cex=1.5)
    # traça a distribuição da média amostral com erro padrão calculado em função do
    # tamanho da amostra e do desvio padrão da distribuição na população
    curve(dnorm(x,infLimit(), sdError()),xlim=c(min(),max()),main="Limite Inferior", xlab = "", ylab = "", yaxt='n', xaxt='n')
    mtext(paste("N(",sprintf("%.1f", infLimit()),", ",sprintf("%.1f", sdError()*sdError()),")", sep=""), cex = 1.5)
    axis(1, at=c(sampleMean()), lwd.ticks = 2)
    # especifica as coordenadas da área correspondente à cauda superior (acima da média amostral)
    # para o limite inferior especificado pelo usuário
    cord.x <- c(sampleMean(),seq(sampleMean(),max(),resolution),max()) 
    cord.y <- c(0,dnorm(seq(sampleMean(),max(),resolution), infLimit(), sdError()),0) 
    # calcula a área da cauda superior, acima da média da amostra,
    # da distribuição da média amostral quando a média da distribuição é 
    # igual ao limite inferior
    p <- pnorm(sampleMean(), infLimit(), sdError(), lower.tail = FALSE)
    # se a área é igual a alfa/2, então traça uma linha vertical azul
    # demarcando o limite inferior do intervalo de confiança, faz uma marca no eixo x
    # e desenha a área em verde
    # caso contrário, só desenha a área em vermelho
    color <- if (abs(p - half.alfa()) < tolerance) {
                par(col.axis = "blue")
                axis(1, at=c(infLimit()), lwd.ticks = 2)
                abline(v=infLimit(), lwd = "2", col = "blue")
                rgb(0, 1, 0,0.5)
             } else {
                rgb(1, 0, 0,0.5)
             }
    polygon(cord.x, cord.y, col=color)
    limits <- par("usr")
    ypos <- 3* par("usr")[4]/4 
    xpos <- sampleMean() + (max() - sampleMean())/4
    # escreve o valor da área da cauda superior
    text(xpos,  ypos, paste("alfa/2 = ", sprintf("%.4f", p)), pos = 4)
  })
}

ui <- fluidPage(
  
  titlePanel(expression("Limites do intervalo de confiança")),
  
  sidebarLayout(
    sidebarPanel(
    sliderInput("dpNormal", 
                 label = expression("Desvio padrão da população"), 
                 min = 1, max = 20,
                 value = 10
    ),
    sliderInput("conflevel", 
                 label = expression("Nível de confiança (%)"), 
                 min = 0, max = 100, step = 1,
                 value = 95
    ),
    sliderInput("sampleSize", 
                 label = "Tamanho Amostral", 
                 min = 1, max = 100,
                 value = 1
    ),
    sliderInput("sampleMean", 
                label = expression("Média amostral"), 
                min = 10, max = 100, step = 2,
                value = 50
    ),
    numericInput("supLimit", 
                 label = "Limite superior", 
                 min = 30, max = 100, step = 1,
                 value = 50
    ),
    numericInput("infLimit", 
                label = "Limite Inferior", 
                min = -10, max = 100, step = 1,
                value = 50
    )
    ),
    
    mainPanel(
      fluidRow(
        column(10, plotOutput("superiorLimit")),
        column(10, plotOutput("inferiorLimit"))
      ))
  )
)


shinyApp(ui = ui, server = server)