# application that shows the reasoning behind the Analysis of Variance
# autor: Sergio Miranda Freire

library(shiny)

server <- function(input, output) {
    mean1 <- reactive ({
        input$mean1
    })
    
    mean2 <- reactive ({
        input$mean2
    })
    
    mean3 <- reactive ({
        input$mean3
    })
    
    sdcommon <- reactive ({
        input$sdcommon
    })
    
    n1 <- reactive ({
        input$n1
    })
    
    n2 <- reactive ({
        input$n2
    })
    
    n3 <- reactive ({
        input$n3
    })
    
    alpha <- reactive ({
        input$alpha/100
    })
    
    x1 <- reactive ({
        rnorm(n1(), mean1(), sdcommon())
    })
    
    x2 <- reactive ({
        rnorm(n2(), mean2(), sdcommon())
    })
    
    x3 <- reactive ({
        rnorm(n3(), mean3(), sdcommon())
    })
    
    grupo <- reactive ({
        as.factor(c(rep(1, n1()), rep(2, n2()), rep(3, n3())))
    })
    
    n <- reactive ({
        n1() + n2() + n3()
    })
    
    x <- reactive ({
        c(x1(), x2(), x3())
    })
    
    x_ <- reactive ({
        mean(x())
    })
    
    x1_ <- reactive ({
        mean(x1())
    })
    
    x2_ <- reactive ({
        mean(x2())
    })
    
    x3_ <- reactive ({
        mean(x3())
    })
    
    data <- reactive ({
        data.frame(x(), grupo())
    })
    
    variance_table <- reactive ({
        SQR <- sum(x() ^ 2) - sum(n1() * x1_() ^ 2, n2() * x2_() ^ 2, n3() * x3_() ^ 2)
        QMR <- SQR / (n() - 3)
        SQT <- sum(x() ^ 2) - n() * x_() ^ 2
        SQG <- sum(n1() * x1_() ^ 2, n2() * x2_() ^ 2, n3() * x3_() ^ 2) - n() * x_()^2
        QMG <- SQG / 2
        F_ <- QMG / QMR
        F_label <- if (F_ > qf(1-alpha(),2, n() - 3)) {
            "F*"
        } else {
            "F"
        }
        
        sqt <- paste(sprintf("%.2f", SQT))
        sqr <- paste(sprintf("%.2f", SQR))
        qmr <- paste(sprintf("%.2f", QMR))
        sqg <- paste(sprintf("%.2f", SQG))
        qmg <- paste(sprintf("%.2f", QMG))
        f <- paste(sprintf("%.2f", F_))
        variance_components_table <- matrix(nrow = 0, ncol = 5)
        gl1 <- paste("n - r = ", sprintf("%3d", n() - 3))
        gl2 <- paste("r - 1 = 2")
        gl3 <- paste("n - 1 = ", sprintf("%3d", n() - 1))
        component1 <- "Residual"
        component2 <- "Entre grupos"
        component3 <- "Total"
        text1 <- c(component1, gl1, sqr, qmr, f)
        variance_components_table <-
            rbind(variance_components_table, text1)
        text2 <- c(component2, gl2, sqg, qmg, "")
        variance_components_table <-
            rbind(variance_components_table, text2)
        text3 <- c(component3, gl3, sqt, "", "")
        variance_components_table <-
            rbind(variance_components_table, text3)
        colnames(variance_components_table) <-
            c(
                "Fonte de variação",
                "Graus de liberdade",
                "Soma dos quadrados",
                "Quadrado médio",
                F_label
            )
        variance_components_table
    })
    
    output$mstot <- renderPlot({
        plot(
            x(),
            col = grupo(),
            main = "Soma dos quadrados total",
            xlab = "",
            ylab = "",
            ylim = c(35, 85)
        )
        abline(h = x_(), lwd = 2)
        segments(1:n(), x(), 1:n(), x_(), col = grupo())
    })
    
    output$mse <- renderPlot({
        plot(
            x(),
            col = grupo(),
            main = "Soma dos quadrados residual",
            xlab = "",
            ylab = "",
            ylim = c(35, 85)
        )
        segments(1, x1_(), n1(), x1_(), col = "black")
        segments(1 + n1(), x2_(), n1() + n2() , x2_(), col = "red")
        segments(1 + n1() + n2(), x3_(), n(), x3_(), col = "green")
        segments(1:n1(), x1(), 1:n1(), x1_(), col = "black")
        segments((1 + n1()):(n1() + n2()),
                 x2(),
                 (1 + n1()):(n1() + n2()),
                 x2_(),
                 col = "red")
        segments((1 +  n1() + n2()):n(), x3(), (1 + n1() + n2()):n(), x3_(), col = "green")
    })
    
    output$msg <- renderPlot({
        plot(
            x(),
            col = grupo(),
            main = "Soma dos quadrados entre grupos",
            xlab = "",
            ylab = "",
            ylim = c(35, 85)
        )
        abline(h = x_(), lwd = 2)
        segments(1, x1_(), n1(), x1_(), col = "black")
        segments(1 + n1(), x2_(), n1() + n2() , x2_(), col = "red")
        segments(1 + n1() + n2(), x3_(), n(), x3_(), col = "green")
        segments(1:n1(), x1_(), 1:n1(), x_(), col = "black")
        segments((1 + n1()):(n1() + n2()),
                 x2_(),
                 (1 + n1()):(n1() + n2()),
                 x_(),
                 col = "red")
        segments((1 +  n1() + n2()):n(), x3_(), (1 + n1() + n2()):n(), x_(), col = "green")
    })
    
    output$variance_components <-
        renderTable({
            variance_table()
        }, align = 'r')
}


ui <- fluidPage(# Application title
    titlePanel(paste(
        expression(Análise), " de ", expression(Variância)
    )),
    
    sidebarLayout(
        sidebarPanel(
            sliderInput(
                "mean1",
                label = paste(expression(Média), "do grupo 1"),
                min = 50,
                max = 70,
                value = 59
            ),
            sliderInput(
                "mean2",
                label = paste(expression(Média), "do grupo 2"),
                min = 50,
                max = 70,
                value = 58
            ),
            sliderInput(
                "mean3",
                label = paste(expression(Média), "do grupo 3"),
                min = 50,
                max = 70,
                value = 60
            ),
            sliderInput(
                "sdcommon",
                label = paste("Desvio", expression(padrão)),
                min = 1,
                max = 10,
                value = 4
            ),
            sliderInput(
                "n1",
                "tamanho da amostra 1",
                min = 5,
                max = 30,
                value = 11
            ),
            sliderInput(
                "n2",
                "tamanho da amostra 2",
                min = 5,
                max = 30,
                value = 10
            ),
            sliderInput(
                "n3",
                "tamanho da amostra 3",
                min = 5,
                max = 30,
                value = 12
            ),
            sliderInput(
                "alpha",
                expression("nível de significância"),
                min = .01,
                max = 100,
                value = 5
            )
        ),
        
        # Show plots
        mainPanel(fluidRow(
            column(5, plotOutput("mse")),
            column(5, plotOutput("msg"))
        ),
        fluidRow(
            column(5, plotOutput("mstot")),
            column(5, align = "center", tableOutput("variance_components"))
        ))
    ))

shinyApp(ui = ui, server = server)