#
# applicação que mostra a construção da curva ROC
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)
resolution <- 0.1

server <- function(input, output) {

  # ponto de corte para o cálculo da especificidade e sensibilidade
  cutoff <- reactive ({
    input$cutoff
  })

  # nome da variável de teste
  variavel <- reactive ({
    input$variavel
  })
  
  # média da distribuição dos não doentes
  mean_inf <- reactive ({
    input$meanNormal
  })
  
  # desvio padrão da distribuição dos não doentes
  sd_inf <- reactive ({
    input$dpNormal
  })
  
  # média da distribuição dos doentes
  mean_sup <- reactive ({
    input$meanDoente
  })
  
  # desvio padrão da distribuição dos doentes
  sd_sup <- reactive ({
    input$dpDoente
  })
  
  # limite inferior para a construção dos gráficos
  inf_limit <- reactive ({
    mean_inf() - 4 * sd_inf()
  })
  
  # limite superior para a construção dos gráficos
  sup_limit <- reactive ({
    mean_sup() + 4 * sd_sup()
  })
  
  output$glicemia <- renderPlot({
    adjX <- (mean_inf() - inf_limit()) / (sup_limit() - inf_limit())
    # coordenadas para o desenho da área correspondendo aos falsos positivos
    cord.x <- c(cutoff(),seq(cutoff(),sup_limit(),resolution),sup_limit()) 
    cord.y <- c(0,dnorm(seq(cutoff(),sup_limit(),resolution), mean_inf(), sd_inf()),0)
    # encontra o valor máximo no eixo y para traçar as duas curvas normais em uma mesma escala
    ymax = max(dnorm(mean_inf(),mean_inf(),sd_inf()), dnorm(mean_sup(),mean_sup(),sd_sup()))
    # gráfico da densidade de probabilidade dos não doentes
    curve(dnorm(x,mean_inf(),sd_inf()),xlim=c(inf_limit(),sup_limit()), main=paste("N(",sprintf("%.1f", mean_inf()),", ",sprintf("%.1f", sd_inf()*sd_inf()),")", sep=""), adj=adjX, xlab = "", ylab = "", xaxt = 'n', yaxt='n', ylim = c(0, ymax), col = "blue", lwd = 2) 
    # desenho da área correspondente aos falsos positivos
    polygon(cord.x,cord.y,col=rgb(1,1, 0,0.5))
    par(new=TRUE)
    adjX <- (mean_sup() - inf_limit()) / (sup_limit() - inf_limit())
    # gráfico da densidade de probabilidade dos doentes
    curve(dnorm(x,mean_sup(),sd_sup()),xlim=c(inf_limit(),sup_limit()),main="", xlab = "", ylab = "", yaxt='n', ylim = c(0, ymax), col = "red", lwd = 2)
    par(cex=1.5)
    mtext(paste("N(",sprintf("%.1f", mean_sup()),", ",sprintf("%.1f", sd_sup()*sd_sup()),")", sep=""), adj= adjX, cex = 1.2, font = 2)
    title(xlab=variavel(), adj = 0.4)
    # coordenadas para o desenho da área correspondendo aos falsos negativos
    cord.z <- c(inf_limit(),seq(inf_limit(),cutoff(),resolution),cutoff())
    cord.w <- c(0,dnorm(seq(inf_limit(),cutoff(),resolution), mean_sup(), sd_sup()),0)
    # desenho da área correspondente aos falsos negativos
    polygon(cord.z,cord.w,col=rgb(0, 1, 0,0.5))
    limits <- par("usr")
    ypos <- 5* par("usr")[4]/6 # obtém 5/6 do valor máximo do eixo y
    xpos <- mean_sup() + (sup_limit() - mean_sup())/5
    par(cex=1.1)
    legend(xpos, ypos, c("Falsos Positivos","Falsos Negativos"), fill=c("yellow","green"))
  })

  output$roc <- renderPlot({
    # gráfico da curva ROC
    par(mar=c(5,5,1,2))
    curve(pnorm(qnorm(x, mean_inf(), sd_inf(), lower.tail=FALSE), mean_sup(), sd_sup(), lower.tail=FALSE) ,xlim=c(0,1),main="", xlab = "", ylab = "", lwd = 2)
    # especificidade e sensibilidade correspondentes ao ponto de corte
    x <- pnorm(cutoff(),mean_inf(), sd_inf(), lower.tail=FALSE)
    y <- pnorm(cutoff(),mean_sup(), sd_sup(), lower.tail=FALSE)
    # plotando o ponto (x, y) e mostrando os valores de sensibilidade e especificidade
    points(x, y, col = "blue", pch=21, cex = 4.0, bg = "blue")
    text(x +.01, y -.05, paste("S = ", sprintf("%.2f", y)), pos = 4)
    text(x +.01, y -.1, paste("E = ", sprintf("%.2f", 1-x)), pos = 4)
    par(cex=1.5)
    title(xlab = "1 - Especificidade", ylab = "Sensibilidade")
    })
}


# Define UI for application that draws a roc curve
ui <- fluidPage(
  
  # Application title
  titlePanel("Curva ROC"),
  
  # Sidebar with a slider input for the cutoff
  sidebarLayout(
    sidebarPanel(
      sliderInput("meanNormal", 
                 label = expression("Média da população Normal"), 
                 min = 60, max = 110,
                 value = 92
    ),
    sliderInput("dpNormal", 
                 label = expression("Desvio-padrão da população Normal"), 
                 min = 6, max = 50,
                 value = 16
    ),
    sliderInput("meanDoente", 
                 label = expression("Média da populacao Doente"), 
                 min = 110, max = 180,
                 value = 140
    ),
    sliderInput("dpDoente", 
                 label = expression("Desvio-padrão da população Doente"), 
                 min = 6, max = 50,
                 value = 23
    ),
    textInput("variavel", label = expression("Variável"), 
              value = "Glicemia"),
    
    sliderInput("cutoff",
                  "Ponto de corte:",
                  min = 40,
                  max = 210,
                  value = 110)
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("glicemia")),
        column(8, plotOutput("roc"))
      ))
  )
)


shinyApp(ui = ui, server = server)