# aplicação que mostra o conceito de densidade de probabilidade
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(DT)

server <- function(input, output) {
  
  # número de elementos da amostra    
  n <- reactive ({
      input$n
  })
  
  # número de classes do histograma    
  classes <- reactive({
    input$breaks
  })

  # gera uma amostra com n elementos da distribuição N(70, 100)
  y <- reactive ({
      set.seed(333)
      rnorm(n(), mean = 70, sd = 10)
  })
  
  # gera o histograma a partir da amostra y e o número de classes
  h <- reactive({
      hist(y(), breaks = classes(), plot = FALSE)
  })
  
  # gera a tabela de frequências a partir da amostra y e do número de classes
  freq_table <- reactive ({
       max <- max(h()$counts)
       indices_max <- h()$counts == max
       indmax <- which(indices_max)
       table <- matrix(nrow = 0, ncol = 4)
       table <- rbind(table, c(" ... ", " ...", "...", "..."))
       for (i in (indmax-3) : (indmax+2)) {
           class <- paste(sprintf("%.2f", h()$breaks[i]), "-", sprintf("%.2f", h()$breaks[i+1]))
           relativeFrequency <- h()$counts[i]/n()
           relFreqString <- paste(sprintf("%.6f", relativeFrequency))
           relFreqDensityString <- paste(sprintf("%.5f", h()$density[i]))
           cumRelFreq <- cumsum(h()$counts)[i]/n()
           cumFreqString <- paste(sprintf("%.4f", cumRelFreq))
           text <- c(class, relFreqString, relFreqDensityString, cumFreqString)
           table <- rbind(table, text)
       }
       table <- rbind(table, c(" ... ", " ...", "...", "..."))
       colnames(table) <- c("Classe", "Frequência Relativa", "Densidade de Frequência", "Densidade de Frequência Acumulada")
       rownames(table) <- NULL
       table
  })
  
   # mostrando a tabela de frequência  
   output$frequencyTable <- DT::renderDataTable({freq_table()}, options = list(
       autoWidth = TRUE, rownames = FALSE, dom = 't', 
       rowCallback = JS("function(r,d) {$(r).attr('height', '30px')}"),
       columnDefs = list(list(width = '20%', targets = 0), list(targets=0:3, class="dt-right"),
                         list(width = '15%', targets = 1:3))
   ))
       
   # plotando o histograma de frequência relativa  
   output$relativeFrequencyHistogram <- renderPlot({
      relh <- h()
      relh$counts <- relh$counts/n()
      plot(relh, ylim=c(0, 0.35), ylab="Frequência relativa", main="Histograma de frequência relativa", xlab = "")
    })

  # plotando o histograma de densidde de frequência  
  output$densityHistogram <- renderPlot({
     plot(h(), freq = FALSE, xlab = "", ylab="Densidade de frequência", main="Histogram de Densidade de Frequência")
  })
  
  # plotando o histograma de densidade de frequência acumulada  
  output$cumulativeRelativeFrequencyHistogram <- renderPlot({
      cumh <- h()
      cumh$counts <- cumsum(cumh$counts)/n()
      plot(cumh, xlab = "", ylab="Densidade de Frequência Acumulada", main="Histograma de Densidade de Frequência Acumulada")
   })
  
}


ui <- fluidPage(
  
  titlePanel(paste("Densidade de", expression(Frequência), "e", expression(Distribuição), "Acumulada", sep = " ")),
  
  sidebarLayout(
    sidebarPanel(
     numericInput("breaks", 
                label = paste(expression(Número), "de classes:"), 
                min = 10, max = 10000,
                value = 10
     ),
     sliderInput("n", 
                label = paste(expression(Número), "de pontos:"), 
                min = 1000, max = 40000000,
                value = 20000000
     ),
     submitButton("Submeter")
     ),
    
    # Show plots
    mainPanel(
      fluidRow(
          column(5, align="center", div(dataTableOutput("frequencyTable"), style = "font-size:90%")),
          column(5, plotOutput("relativeFrequencyHistogram"))
      ),
      fluidRow(
        column(5, plotOutput("densityHistogram")),
        column(5, plotOutput("cumulativeRelativeFrequencyHistogram"))
      )
    )
  )
)


shinyApp(ui = ui, server = server)