#
# aplicação que mostra o conceito de derivada
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#


library(shiny)
library(plotrix)

# Distribuição normal que será utilizada N(80, 400)
popMean <- 80
popSd <- 20
minAxis <-  popMean - 1.3 * popSd
maxAxis <- popMean - 0.75 * popSd

server <- function(input, output) {
  
  # número de pontos que serão usados para dividir o intervalo do gráfico 
  n <- reactive ({
    input$n
  })

  
  # plota a função densidade de probabilidade e os dois segmentos de reta que passam pelos pontos
  # [(a, F(a)), ((a + dx, F(a+dX)))] e [(xcenter, F(xcenter)), ((xcenter + dx, F(xcenter+dX)))]
  output$derivada <- renderPlot({
    # limites inferior e superior do eixo X
    a <- popMean - 1.2 * popSd
    b <- popMean - .8 * popSd
    # número de intervalos em que o gráfico será dividido
    delta <- (b - a) / n()
    ymax = (pnorm(b,popMean, popSd, lower.tail = TRUE))^6
    # gráfico da função de distribuição acumulada de probabilidade N(80, 400)
    curve((pnorm(x,popMean, popSd, lower.tail = TRUE))^6,xlim=c(minAxis, maxAxis+1), ylim=c(-0.00002, 0.00012), xlab = "", ylab = "", xaxt = "n", yaxt = "n", axes = FALSE, lwd = 2) 
    axis(1, pos=0, labels = FALSE)
    axis(2, pos=54, labels = FALSE)
    axesLim <- par("usr")
    # coordenadas dos pontos da distribuição acumulada de probabilidade N(80, 400)
    cord.x <- c(a, a)
    Fx <- pnorm(a, popMean, popSd, lower.tail = TRUE)^6
    cord.y <- c(0, Fx, Fx)
    for ( i in c(1:n())) {
      cord.x <- c(cord.x, rep(a + i * delta, 2))
      cord.y <- c(cord.y, rep((pnorm(a + i * delta, popMean, popSd, lower.tail = TRUE))^6, 2))
    }
    k <- 2*n()+2
    center <- n() + 2
    cord.y <- c(cord.y[1:k],0)
    cord.x <- c(cord.x, b)

    escalaConversion <- (cord.y[k] - cord.y[2]) * 1.3 / (b-a)
    
    # mostra dois segmentos de reta usando as duas primeiras coordenadas calculadas acima 
    # e as duas coordenadas no centro do gráfico plotado
    beta1 <- (cord.y[4] - Fx) / delta
    beta2 <- (cord.y[center + 2] - cord.y[center] ) / delta
    alfa1 <- Fx - beta1 * a
    alfa2 <- cord.y[center + 2] - beta2 * cord.x[center + 2]
    segments(a-1, alfa1 + beta1 * (a-1), (a + b + 1) /2, alfa1 + beta1 * (a + b + 1) /2, col = "red", lwd = 2)
    segments((a+b)/2 , alfa2 + beta2 *(a+b)/2, b+ 1, alfa2 + beta2 * (b + 1), col = "green", lwd = 2)

    par(cex=1.2)
    # para menos de 8 intervalos, mostra os incrementos no eixo x e eixo y
    if (n() < 8) {
        draw.arc(cord.x[center], cord.y[center], delta / 2, angle1 = 0, angle2 = atan(beta2/escalaConversion), col = "blue", lwd = 2)
        segments(cord.x[center + 2] + delta / 8, cord.y[center], cord.x[center + 2] + delta / 8,  cord.y[center + 2], col= "blue", lwd = 2)
        segments(cord.x[center + 2] + delta / 8 - .2, cord.y[center], cord.x[center + 2] + delta/8 + .2, cord.y[center], col = "blue", lwd = 2)
        segments(cord.x[center + 2] + delta/8 - .2, cord.y[center+2], cord.x[center + 2]+ delta/8 + .2, cord.y[center+2], col = "blue", lwd = 2)
        segments(cord.x[center], cord.y[center] -0.000005, cord.x[center + 2],  cord.y[center] -0.000005, col= "blue", lwd = 2)
        segments(cord.x[center], cord.y[center]-0.000007, cord.x[center], cord.y[center]-0.000003, col = "blue", lwd = 2)
        segments(cord.x[center+2], cord.y[center]-0.000007, cord.x[center + 2], cord.y[center]-0.000003, col = "blue", lwd = 2)
        text((cord.x[center] + cord.x[center+2]) / 2, cord.y[center]-0.000008, expression(paste(Delta, "x")))
        text(cord.x[center + 2]+ delta / 3, (cord.y[center] + cord.y[center+2])/2, labels=expression(paste(Delta, "F")))
        text(cord.x[center], -0.000005, labels=expression(paste("x"[0])))
        text(cord.x[center + 2], -0.000005, labels=expression(paste("x"[0]," + ", Delta, "x")))
        text(cord.x[center] +  (cord.x[center+2] - cord.x[center])* 2 / 3, cord.y[center] + (cord.y[center+2] - cord.y[center]) /4, expression(Theta))
        text(cord.x[center]-.5, cord.y[center] + 0.000005, labels=expression(paste("F(x"[0], ")")))
        text(cord.x[center+2]-.5, cord.y[center+2] + 0.000005, labels=expression(paste("F(x"[0],"+", Delta, "x)")))
    } else {
        text(cord.x[center], -0.000005, labels=expression(paste("x"[0])))
        text(cord.x[center]-.5, cord.y[center] + 0.000005, labels=expression(paste("F(x"[0], ")")))
    }
    if (n() == 200) { # se n = 200 substitui a divisão pela diferenciação
        text(58, cord.y[k], expression(paste("tg ", Theta, " = ", frac(paste("dF"), paste("dx")), "  =   lim  ", frac(paste("F(x"[0],"+", Delta, "x) - F(x)"),paste(Delta, "x")))))
        par(cex=1)
        text(58, cord.y[k]-0.000007, expression(paste(" ", Delta, "x -> 0")))
    } else {
        text(58, cord.y[k], expression(paste("tg ", Theta, " = ", frac(paste(Delta, "F"), paste(Delta, "x")), " = ", frac(paste("F(x"[0],"+", Delta, "x) - F(x)"),paste(Delta, "x")))))
    }
  })
}


ui <- fluidPage(
  
  titlePanel("Derivada"),
  
  sidebarLayout(
    sidebarPanel(
      numericInput("n", 
                label = expression("Número de pontos"), min = 2, max = 200, step = 2, value = 2
      )
    ),
    
    mainPanel(
    plotOutput("derivada", width = 500, height = 500))
  )
)


shinyApp(ui = ui, server = server)