#
# aplicação que mostra a distribuicao T para diversos tamanhos amostrais e distribuições normais 
# da variável aleatória original
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

server <- function(input, output) {
  
  # Média da distribuição normal   
  meanNormal <- reactive ({
      input$mean
  })

  # Desvio padrão da distribuição normal   
  sdNormal <- reactive ({
      input$stdev
  })

  # número de amostras
  sampleCount <- reactive ({
    input$sampleCount
  })
  
  # Tamanho de cada amostra
  sampleSize <- reactive ({
    input$sampleSize
  })


  # número de observações
  npoints <- reactive({
      sampleSize() * sampleCount()
  })

  # número de classes no histograma 
  classes <- reactive({
    input$breaks
  })
  
  # Distribuição normal
  yNormal <- reactive ({
    rnorm(npoints(), mean=meanNormal(), sd=sdNormal())
  })

  # traça o gráfico da distribuição normal
  output$distribution <- renderPlot({
      inf_limit <- meanNormal() - 4 * sdNormal()
      sup_limit <- meanNormal() + 4 * sdNormal()
      curve(dnorm(x, meanNormal(), sdNormal()), xlim = c(inf_limit, sup_limit),
                  main = paste("N(", sprintf("%.1f", meanNormal()), ", ", sprintf("%.1f", sdNormal() ^ 2), ")", sep = ""),
                  xlab = "", ylab = "")
  })
  
  # plota o histograma da variável (média amostral - mu)/(s/sqrt(n)) superposto à  distribuição T
  output$sampleMeanDensityHistogram <- renderPlot({
    # x é uma matriz de duas dimensões (tamanho da amostra, número de amostras)
    y <- matrix(yNormal(), nc = sampleCount())
    #   valor mínimo do eixo x
    #   valor máximo do eixo x
    min_xaxis <- qt(.0001, df = sampleSize() - 1)
    max_xaxis <- - min_xaxis
    # ybar é um vetor que contém a média de cada amostra da matriz y
    ybar <- apply(y, 2, mean)
    # sdError é o erro padrão de cada amostra
    sdError <- apply(y, 2, sd)/sqrt(sampleSize())
    # valores de t para cada amostra    
    t <- (ybar - meanNormal()) / sdError
    h <- hist(t, freq=FALSE, breaks = classes(), plot = FALSE)
    # máximo valor do histograma ou da distribuição t
    yHmax <- max(h$density)
    ytmax <- dt(0, df = sampleSize() - 1)
    ymax <- max(yHmax, ytmax)
    limits <- par("usr")
    ypos <- par("usr")[4]/2 # obtém a metade do valor máximo do eixo y
    xpos <- min_xaxis + (max_xaxis - min_xaxis)/2
    par(cex=1.0)
    # valor de t que delimita a área de 0.025 na cauda inferior
    cutoff1 <- qt(0.025, sampleSize() - 1)
    # valor de t que delimita a área de 0.025 na cauda superior
    cutoff2 <- - cutoff1
    x <- seq(min_xaxis, max_xaxis, 0.01)
    # curva t
    plotDistr(x, dt(x, sampleSize() - 1), xlab="", ylab="", main="Distribuição t", xlim=c(min_xaxis, max_xaxis), 
              region=list(c(min_xaxis, cutoff1), c(cutoff2, max_xaxis)), col=rgb(1, 0, 0,0.5), legend = FALSE, lwd = "2")
    hist(t, freq=FALSE, breaks = classes(), xlab = "", ylab=expression("Densidade de Frequência"), 
         xlim=c(min_xaxis, max_xaxis), ylim = c(0, ymax), add = TRUE)
    ypos <- ymax*3/4 # 75% do valor máximo do eixo y
    xpos <- max_xaxis/3
    par(cex=1.2)
    legend(xpos, ypos, paste("t(0.975,", sprintf("%d", sampleSize() - 1), ") = ", sprintf("%.2f", cutoff2)), fill=c("red"))
  })
}

ui <- fluidPage(
  
  titlePanel(expression("Distribuição t de Student")),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("mean", 
                      expression("Média da distribuição normal"), 
                      min = 0, max = 100, value = 10
      ),
      sliderInput("stdev", 
                      expression("Desvio padrão da distribuição normal"), 
                      min = 0, max = 50,
                      value = 1
      ),
      sliderInput("sampleSize", 
                      "Tamanho amostral", 
                      min = 5, max = 200,
                      value = 2
      ),
      sliderInput("sampleCount", 
                  expression("Número de amostras"), 
                  min = 1000, max = 100000,
                  value = 10000
      ),
      sliderInput("breaks", 
                  expression("Número de classes no histograma:"), 
                  min = 20, max = 200,
                  value = 20
      ),
      submitButton("Submeter")
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("distribution")),
        column(8, plotOutput("sampleMeanDensityHistogram"))
      )
    )
  )
)

shinyApp(ui = ui, server = server)