#
# aplicação que mostra os conceitos de erros tipo I e II
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

# média sob a hipótese nula
meanH0 <- 80

# desvio padrão
sd <- 16

# tamanho amostral
sampleSize <- 25

# erro padrão
sdError <- sd / sqrt(sampleSize)

# valores mínimo e máximo do eixo X
resolution <- 0.01
min <- qt(.001, sampleSize-1, ncp = -3.75, lower.tail = TRUE)
max <- qt(.001, sampleSize-1, ncp = 3.75, lower.tail = FALSE)

# sequência de pontos no eixo X
x <- seq(min, max,resolution)

server <- function(input, output) {
    
  # metade do erro alfa
  half.alfa <- reactive ({
    input$alfa / 200
  })
  
  # média sob a hipótese alternativa
  meanH1 <- reactive ({
    input$meanH1
  })

  # diferença de médias / erro padrão
  diff <- reactive ({
      (meanH1() - meanH0) / sdError
  }) 
  
  # ponto crítico inferior
  cutoff1 <- reactive ({
    qt(half.alfa(), sampleSize-1, lower.tail = TRUE)
  })
  
  # ponto crítico superior
  cutoff2 <- reactive ({
      qt(1-half.alfa(), sampleSize-1, lower.tail = TRUE)
  })
  
  # plota as distribuições com as respectivas áreas 
  # correspondentes aos erros tipo I e II
  output$distributions <- renderPlot({
    adjX <-  (0-min)/ (max - min)
    # plota a distribuição correspondente à hipótese nula e a região correspondente a alfa
    plotDistr(x, dt(x, sampleSize - 1), xlab="", ylab="", main="H0", xlim=c(min, max), adj = adjX, xaxt = 'n', yaxt = 'n',
              region=list(c(min, cutoff1()), c(cutoff2(), max)), col=rgb(1, 0, 0,0.5), legend = FALSE)
    par(new=TRUE)
    adjY <- abs(diff() - min) / (max - min)
    # plota a distribuição correspondente à hipótese alternativa
    if(abs(meanH0 - meanH1()) > 2) {  # este teste é feito para evitar a superposição dos rótulos H0 e H1 no gráfico
        plotDistr(x, dt(x-diff(), sampleSize - 1), xlab="", ylab = "", main = "H1", xlim = c(min, max), adj = adjY,
                  region = list(c(cutoff1(), cutoff2())), col = rgb(1, 1, 0,0.5), xaxt = 'n', yaxt = 'n', legend = FALSE)
    } else {
        plotDistr(x, dt(x-diff(), sampleSize - 1), xlab="", ylab="", main="", xlim=c(min, max), adj=adjY,
                  region = list(c(cutoff1(), cutoff2())), col=rgb(1, 1, 0,0.5), xaxt = 'n', yaxt='n', legend = FALSE)
        mtext("H1", adj= adjY, cex = 1.2, font = 2)
    }
    # linha marcando a média sob a hipótese alternativa
    abline(v = diff(), col = "blue")
    limits <- par("usr")
    ypos <- 7 * par("usr")[4]/8 # 88% do valor máximo do eixo y
    xpos <- min
    par(cex=1.2)
    title(ylab = "Densidade de Probabilidade")
    par(cex=1.2)
    legend(xpos, ypos, c(expression(paste("Erro ", alpha)),expression(paste("Erro ", beta))), fill=c("red","yellow"))
  })

  # plota o gráfico do poder estatístico
  output$power <- renderPlot({
    par(mar=c(5,5,1,2))
    # gráfico do poder estatístico em função do valor da média sob a hipótese alternativa
    curve(pt(cutoff2(), sampleSize - 1, ncp = x, lower.tail=TRUE) - pt(cutoff1(), sampleSize - 1, ncp = x, 
                        lower.tail=TRUE), xlim=c(min, max), xaxt='n', xlab = "", ylab = "", col = "black")
    # valor do poder estatístico para o valor de média sob a hipótese alternativa 
    # selecionada pelo usuário
    valor_x <- diff()
    y <- pt(cutoff2(), sampleSize - 1, ncp = diff(), lower.tail=TRUE)-pt(cutoff1(), sampleSize - 1, ncp = valor_x, lower.tail=TRUE)
    # linha marcando a média sob a hipótese alternativa
    abline(v = valor_x, col = "blue")
    # plota o ponto em azul no gráfico do poder estatístico para a média sob a hipótese nula 
    # selecionada pelo usuário
    points(valor_x, y, col = "blue", pch=21, cex = 2.0, bg = "blue")
    # escreve o valor do poder estatístico para a média selecionada
    text(valor_x , y -.05, paste(expression(beta),' = ', sprintf("%.2f", y)), pos = 4)
    par(cex=1.2)
    title(xlab = "H1", ylab = expression(paste("Erro ", beta)))
  })
}

ui <- fluidPage(
  
  titlePanel("Erros tipo I e tipo II"),
  
  sidebarLayout(
    sidebarPanel(
    sliderInput("meanH1", 
                 label = expression("Média da populacao sob H1"), 
                 min = 73, max = 97, value = 92),
    sliderInput("alfa",
                  expression("Nível de significancia (%)"),
                  min = 1, max = 100, value = 5)
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("distributions")),
        column(8, plotOutput("power"))
      ))
  )
)

shinyApp(ui = ui, server = server)