#
# applicação que mostra a influência da sensibilidade sobre a probabilidade pós-teste
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)

# Função que fornece o valor preditivo positivo
vpp <- function(prev, sens, spec) {
    (prev * sens) / (prev*sens + (1-spec) * (1 - prev))
}

# Função que fornece a probabilidade de ter a doença quando o teste é negativo
probD_test_negative <- function(prev, sens, spec) {
    (prev* (1 - sens) ) / (prev* (1 - sens) + spec * (1 - prev))
}

# Função que fornece a probabilidade de não ter a doença quando o teste é positivo
probNotD_test_positive <- function(prev, sens, spec) {
    (1-prev) * (1-spec) / (prev*sens + (1-spec) * (1 - prev))
}

# Função que fornece o valor preditivo negativo
vpn <- function(prev, sens, spec) {
    (1-prev) * spec  / (prev* (1 - sens) + spec * (1 - prev))
}

# Define server logic required to draw the curve
server <- function(input, output) {

#Parâmetros escolhidos pelo usuário: especificidade, sensibilidade
  specificity <- reactive ({
    input$specificity
  })
  
  sensitivity <- reactive ({
    input$sensitivity
  })

 # Gráfico do valor preditivo positivo
  output$vpp_curve <- renderPlot({
    curve(vpp(x, sensitivity(), specificity()), xlim=c(0,1), main="Teste Positivo", xlab = "", ylab = "", lwd = 2)
    # curvas de referência  
    curve(vpp(x, 0.7, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "green", add = TRUE)
    curve(vpp(x, 0.8, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "red", add = TRUE)
    curve(vpp(x, 0.9, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "blue", add = TRUE)
    par(cex=1)
    title(xlab = "Probabilidade Pré-teste", ylab = "Valor Preditivo Positivo")
    legend(.5, .5, c("S = 0,7", "S = 0,8", "S = 0,9"), fill=c("green", "red", "blue"))
  })
  
  output$pD_testNegative <- renderPlot({
      curve(probD_test_negative(x, sensitivity(), specificity()), xlim=c(0,1), main="Teste Negativo", xlab = "", ylab = "", lwd = 2)
      # curvas de referência  
      curve(probD_test_negative(x, 0.7, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "green", add = TRUE)
      curve(probD_test_negative(x, 0.8, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "red", add = TRUE)
      curve(probD_test_negative(x, 0.9, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "blue", add = TRUE)
      par(cex=1.2,  mar=c(4,5,2,1))
      title(xlab = "Probabilidade Pré-teste", ylab = expression(P(D^"+"/T^"-")))
      legend(.5, .5, c("S = 0,7", "S = 0,8", "S = 0,9"), fill=c("green", "red", "blue"))
  })

  output$vpn_curve <- renderPlot({
      curve(vpn(x, sensitivity(), specificity()), xlim=c(0,1),main="Teste Negativo", xlab = "", ylab = "", lwd = 2)
      # curvas de referência  
      curve(vpn(x, 0.7, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "green", add = TRUE)
      curve(vpn(x, 0.8, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "red", add = TRUE)
      curve(vpn(x, 0.9, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "blue", add = TRUE)
      par(cex=1)
      title(xlab = "Probabilidade Pré-teste", ylab = "Valor Preditivo Negativo")
      legend(.5, .5, c("S = 0,7", "S = 0,8", "S = 0,9"), fill=c("green", "red", "blue"))
  })
  
  output$pNotD_testPositive <- renderPlot({
      curve(probNotD_test_positive(x, sensitivity(), specificity()), xlim=c(0,1), main="Teste Positivo", xlab = "", ylab = "", lwd = 2)
      # curvas de referência  
      curve(probNotD_test_positive(x, 0.7, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "green", add = TRUE)
      curve(probNotD_test_positive(x, 0.8, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "red", add = TRUE)
      curve(probNotD_test_positive(x, 0.9, specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", col = "blue", add = TRUE)
      par(cex=1.2,  mar=c(4,5,2,1))
      title(xlab = "Probabilidade Pré-teste", ylab = expression(P(D^"-"/T^"+")))
      legend(.5, .5, c("S = 0,7", "S = 0,8", "S = 0,9"), fill=c("green", "red", "blue"))
  })
  
}


# Define UI for application that draws a roc curve
ui <- fluidPage(
  
  # Application title
  titlePanel("Influência da Sensibilidade sobre a Probabilidade Pós-Teste"),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("sensitivity", 
                 label = "Sensibilidade", 
                 min = 0, max = 1,
                 value = 0.98
    ),
    sliderInput("specificity", 
                 label = "Especificidade", 
                 min = 0, max = 1,
                 value = 0.75
    )
  ),

    # Show plots
    mainPanel(
        fluidRow(
            column(5, plotOutput("vpp_curve")),
            column(5, plotOutput("pD_testNegative"))
        ),
        fluidRow(
            column(5, plotOutput("pNotD_testPositive")),
            column(5, plotOutput("vpn_curve"))
        )
    )
  )
)


shinyApp(ui = ui, server = server)