#
# aplicação que mostra o intervalo de confiança de uma distribuição normal
# para um certo número de amostras
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(stats)

server <- function(input, output) {

  # número de amostras   
  sampleCount <- reactive ({
    input$sampleCount
  })
  
  # tamanho da amostra   
  n <- reactive ({
    input$sampleSize
  })
  
  # Média da distribuição normal   
  popMean <- reactive ({
    input$mean
  })
  
  # Desvio padrão da distribuição normal   
  popSd <- reactive ({
    input$sd
  })
  
  # Erro padrão da média amostral   
  sdError <- reactive ({
    popSd() / sqrt(n())
  })

  # metade do valor de alfa   
  half.alfa <- reactive ({
    (100 - input$cl) / 200
  })
  
  #   valor mínimo do eixo x
  minAxis <- reactive ({
    popMean() - 4 * popSd()
  })
  
  #   valor máximo do eixo x
  maxAxis <- reactive ({
    popMean() + 4 * popSd()
  })
  
  #  ponto da distribuição da média amostral cuja área à esquerda é alfa/2 
  infCutoff <- reactive ({
    popMean() + qnorm(half.alfa(), mean = 0, sd = 1, lower.tail = TRUE) * sdError()
  })
  
  #  ponto da distribuição da média amostral cuja área à direita é alfa/2 
  supCutoff <- reactive ({
    popMean() + qnorm(half.alfa(), mean = 0, sd = 1, lower.tail = FALSE) * sdError()
   })
  
  output$confidenceIntervals <- renderPlot({
    # total de pontos a serem amostrados
    npoints <- n() * sampleCount()
    # geração de sampleCount amostras de tamanho n da distribuição normal com média e desvio
    # padrão selecionados pelo usuário
    y <- matrix(rnorm(npoints, mean = popMean(), sd = popSd()), nc = sampleCount())
    par(cex = 1.5)
    title = "Intervalos de confiança"
    plot(c(minAxis(), maxAxis()), c(1, sampleCount()), main = title, type = "n", xlab = "", ylab = "Número da amostra")
    # traça as linhas que delimitam áreas iguais a alfa/2 à esquerda e à direita da distribuição normal
    abline(v = infCutoff(), col = "red", lwd = 2)
    abline(h = 0, v = supCutoff(), col = "red", lwd = 2)
    # traça as linha correspondente à média da distribuição normal
    abline(h = 0, v = popMean(), col = "black", lwd = 2)
    for (i in c(1:sampleCount())) {
      x <- y[,i]
      # para cada amostra, obtém a média e os limites do intervalo de confiança   
      sampleMean <- mean(x)
      infCL <- sampleMean + qnorm(half.alfa(), mean = 0, sd = 1, lower.tail = TRUE) * sdError()
      supCL <- sampleMean + qnorm(half.alfa(), mean = 0, sd = 1, lower.tail = FALSE) * sdError()
      # se o intervalo estiver à direita da média da distribuição, 
      # ele será plotado em vermelho
      # se tiver à esquerda da média, será plotado em azul
      # se contiver a média, será plotado em preto
      color <- if (popMean() < infCL) {
        "red"
      } else if (popMean() > supCL) {
        "blue"
      } else {
        "black"
      }
      # traça o intervalo de confiança
      segments(infCL, i, supCL,  i, col= color, lwd = 2)
      # traça um pequeno segmento para indicar a média da amostra
      segments(sampleMean, i - .3, sampleMean, i + .3, col = "black", lwd = 2)
    }
  })
}

ui <- fluidPage(
  
  titlePanel(expression("Intervalos de confiança")),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("mean", 
                label = expression("Média da população"), 
                min = 50, max = 100, step = 1,
                value = 80
    ),
    sliderInput("sd", 
                label = expression("Desvio padrão da população"), 
                min = 10, max = 50, step = 1,
                value = 20
    ),
    sliderInput("cl", 
                 label = expression("Nível de confiança (%)"), 
                 min = 80, max = 100, step = .5,
                 value = 95
    ),
    sliderInput("sampleCount", 
                 label = expression("Número de amostras"), 
                 min = 1, max = 100,
                 value = 20
    ),
    sliderInput("sampleSize", 
                label = "Tamanho da amostra", 
                min = 1, max = 100, step = 1,
                value = 10
    ),
    submitButton("Submeter")
    ),
    
    mainPanel(
      plotOutput("confidenceIntervals", height=800))
  )
)

shinyApp(ui = ui, server = server)