#
# aplicação que mostra o conceito de poder estatístico e a influência do tamanho amostral
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

resolution <- 0.01

# média sob a hipótese nula
meanH0 <- 100

# valor mínimo do eixo X
minx <- meanH0 - 40

# valor máximo do eixo X
maxx <- meanH0 + 40

# sequência de valores do eixo x
x <- seq(minx, maxx,resolution)

server <- function(input, output) {

  # metade do erro alfa
  half.alfa <- reactive ({
    input$alfa / 200
  })
  
  # média sob a hipótese alternativa
  meanH1 <- reactive ({
    input$meanH1
  })

  # desvio padrão da população
  stdev <- reactive({
      input$stdev
  })
  
  # erro padrão das distribuições das médias amostrais
  sdError <- reactive ({
    stdev() / sqrt(input$sampleSize)
  })
  
  # diferença de médias / erro padrão
  diff <- reactive ({
      (meanH1() - meanH0) / sdError()
  }) 
  
  # ponto crítico superior
  critUpperValue <- reactive ({
    qnorm(half.alfa(), meanH0, sdError(), lower.tail = FALSE)
  })
  
  # ponto crítico inferior
  critLowerValue <- reactive ({
      qnorm(half.alfa(), meanH0, sdError(), lower.tail = TRUE)
  })

  # plota as distribuições com as respectivas áreas 
  # correspondentes aos erros tipo I e II
  output$distributions <- renderPlot({
    adjX <- (meanH0 - minx) / (maxx - minx)
    # plota a distribuição correspondente à hipótese nula e a região correspondente a alfa
    plotDistr(x, dnorm(x, meanH0, sdError()), xlab="", ylab="", main="H0", xlim=c(minx, maxx), adj = adjX,
              region=list(c(minx, critLowerValue()), c(critUpperValue(), maxx)), col=rgb(1, 0, 0,0.5), legend = FALSE)
    par(new=TRUE)
    adjY <- (meanH1() - minx) / (maxx - minx)
    # plota a distribuição correspondente à hipótese alternativa
    if(abs(meanH0 - meanH1()) > 2) {  # este teste é feito para evitar a superposição dos rótulos H0 e H1 no gráfico
        plotDistr(x, dnorm(x, meanH1(), sdError()), xlab="", ylab="", main="H1", xlim=c(minx, maxx), adj=adjY,
                  region=list(c(critLowerValue(), critUpperValue())), col=rgb(1, 1, 0,0.5), yaxt='n', legend = FALSE)
    } else {
        plotDistr(x, dnorm(x, meanH1(), sdError()), xlab="", ylab="", main="", xlim=c(minx, maxx), adj=adjY,
              region=list(c(critLowerValue(), critUpperValue())), col=rgb(1, 1, 0,0.5), yaxt='n', legend = FALSE)
        mtext("H1", adj= adjY, cex = 1.2, font = 2)
    }
    title(ylab = "Densidade de Probabilidade")
    par(cex=1.2)
    ypos <- 3 * par("usr")[4]/4 # 75% do valor máximo no eixo Y
    xpos <- meanH0 + (maxx - meanH0)/2
    legend(xpos, ypos, c(expression(paste("Erro ", alpha)), expression(paste("Erro ", beta))), fill=c("red","yellow"))
  })

  # plota o gráfico do poder estatístico
  output$power <- renderPlot({
    par(mar=c(5,5,1,2))
    # gráfico do poder estatístico em função do valor da média sob a hipótese alternativa
    curve(1 - pnorm(critUpperValue(), x, sdError(), lower.tail=TRUE) + pnorm(critLowerValue(), x, sdError(), lower.tail=TRUE), xlim=c(minx, maxx), xlab = "", ylab = "", col = "black")
    # plota o ponto em azul no gráfico do poder estatístico para a média sob a hipótese nula 
    # selecionada pelo usuário
    xpos <- meanH1()
    y <- 1- pnorm(critUpperValue(), meanH1(), sdError(), lower.tail=TRUE) + pnorm(critLowerValue(), meanH1(), sdError(), lower.tail=TRUE)
    points(xpos, y, col = "blue", pch=21, cex = 4.0, bg = "blue")
    # escreve o valor do poder estatístico para a média selecionada
    text(xpos, y -.05, paste("poder = ", sprintf("%.2f", y)), pos = 4)
    par(cex=1.2)
    abline(v=xpos, col = "blue")
    title(xlab = "H1", ylab = expression(paste("Poder Estatístico = 1 - ", beta)))
  })
}

ui <- fluidPage(
  
  titlePanel(expression("Poder Estatístico e Tamanho Amostral")),
  
  sidebarLayout(
    sidebarPanel(
    sliderInput("stdev", 
                 label = expression("Desvio-padrão da população"), 
                 min = 5, max = 30, value = 16
    ),
    sliderInput("meanH1", 
                 label = expression("Média da população sob H1"), 
                 min = 80, max = 120, value = 90
    ),
    sliderInput("sampleSize", 
                 label = "Tamanho amostral", 
                 min = 4, max = 4000, value = 10
    ),
    sliderInput("alfa",
                  expression("Nível de significância (%)"),
                  min = 1, max = 20, value = 5)
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("distributions")),
        column(8, plotOutput("power"))
      ))
  )
)

shinyApp(ui = ui, server = server)