# aplicação que gera uma tabela da razão de verossimilhança para um teste de diagnóstico 
# com uma variável contínua
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)
resolution <- 0.1
health_col <- colorRampPalette(c("white", "green"))
disease_col <- colorRampPalette(c("yellow", "red3"))

server <- function(input, output) {
    
    # número de intervalos em que será dividida a variável de teste
    intervals <- reactive ({
        input$intervals
    })
    
    # nome da variável de teste
    variavel <- reactive ({
        input$variavel
    })
    
    # média da distribuição dos não doentes
    mean_inf <- reactive ({
        input$meanNormal
    })
    
    # desvio padrão da distribuição dos não doentes
    sd_inf <- reactive ({
        input$dpNormal
    })
    
    # média da distribuição dos doentes
    mean_sup <- reactive ({
        input$meanDoente
    })
    
    # desvio padrão da distribuição dos doentes
    sd_sup <- reactive ({
        input$dpDoente
    })
    
    # limite inferior para a construção dos gr?ficos
    inf_limit <- reactive ({
        mean_inf() - 4 * sd_inf()
    })

    # limite superior para a construção dos gráficos
    sup_limit <- reactive ({
        mean_sup() + 4 * sd_sup()
    })
    
    # construção da tabela da razão de verossimilhan?a
    likelihood_tb <- reactive ({
        # intervalo total
        range <- (sup_limit() - inf_limit()) / (intervals())
        region_start <- inf_limit()
        likelihood_tab <- matrix(nrow = 0, ncol = 2)
        # c?lculo da razão de verossimilhança para cada intervalo
        for (i in c(1:intervals())) {
            region_end <- region_start + range
            likelihood_disease <-
                pnorm(region_end, mean_sup(), sd_sup(), lower.tail = TRUE) - pnorm(region_start, mean_sup(), sd_sup(), lower.tail = TRUE)
            likelihood_health <-
                pnorm(region_end, mean_inf(), sd_inf(), lower.tail = TRUE) - pnorm(region_start, mean_inf(), sd_inf(), lower.tail = TRUE)
            text <- paste(sprintf("%.1f", region_start), "-", sprintf("%.1f", region_end), sep = "")
            likelihood_ratio <- likelihood_disease / likelihood_health
            likelihood_tab <- rbind(likelihood_tab, c(text, paste(sprintf("%.2f",likelihood_ratio))))
            region_start <- region_end
        }
        colnames(likelihood_tab) <- c("Intervalos", "Razão de Verossimilhança")
        likelihood_tab
    }
    )
    
    output$glicemia <- renderPlot({
        ymax = max(dnorm(mean_inf(), mean_inf(), sd_inf()), dnorm(mean_sup(), mean_sup(), sd_sup()))
        adjX <- (mean_inf() - inf_limit()) / (sup_limit() - inf_limit())
        # gráfico da densidade de probabilidade dos não doentes
        curve(dnorm(x, mean_inf(), sd_inf()), xlim = c(inf_limit(), sup_limit()),
            main = paste("N(", sprintf("%.1f", mean_inf()), ", ", sprintf("%.1f", sd_inf() * sd_inf()), ")", sep = ""),
            adj = adjX, xlab = "", ylab = "", xaxt = 'n', yaxt = 'n', ylim = c(0, ymax), col = "green", lwd = 3
        )
        par(new = TRUE)
        adjX <- (mean_sup() - inf_limit()) / (sup_limit() - inf_limit())
        # gráfico da densidade de probabilidade dos doentes
        curve(dnorm(x, mean_sup(), sd_sup()), xlim = c(inf_limit(), sup_limit()),
            main = "", adj = adjX, xlab = "", ylab = "", xaxt = 'n', yaxt = 'n', ylim = c(0, ymax), col = "red", lwd = 3
        )
        par(cex = 1.5)
        mtext(paste("N(", sprintf("%.1f", mean_sup()), ", ", sprintf("%.1f", sd_sup() * sd_sup()), ")", sep = ""),
            adj = adjX, cex = 1.2, font = 2
        )
        title(xlab = variavel(), adj = 0.4)
        range <- (sup_limit() - inf_limit()) / (intervals())
        region_start <- inf_limit()
        # faixa de cores para cada uma das distribuições
        health_col_range <- health_col(intervals())
        disease_col_range <- disease_col(intervals())
        # divisão dos intervalos e preenchimento das respectivas áreas
        for (i in c(1:intervals())) {
            region_end <- region_start + range
            cord.x <- c(region_start, seq(region_start, region_end, resolution), region_end)
            cord.y <- c(0, dnorm(seq(region_start, region_end, resolution), mean_inf(), sd_inf()), 0)
            polygon(cord.x, cord.y, col = adjustcolor(health_col_range[i], alpha.f = 0.5))
            cord.w <- c(0, dnorm(seq(region_start, region_end, resolution), mean_sup(), sd_sup()), 0)
            polygon(cord.x, cord.w, col = adjustcolor(disease_col_range[i], alpha.f = 0.5))
            region_start <- region_end
        }
    })
    
    output$likelihood_table <- renderTable({likelihood_tb()}, align = 'r')
}


ui <- fluidPage(
    titlePanel(expression("Razão de Verossimilhança para Variáveis Contínuas")),
    
    sidebarLayout(
        sidebarPanel(
            sliderInput("meanNormal", label = expression("Média da população Normal"),
                min = 60, max = 110, value = 92
            ),
            sliderInput("dpNormal", label = expression("Desvio-padrão da população Normal"),
                min = 6, max = 50, value = 16
            ),
            sliderInput("meanDoente", label = expression("Média da população Doente"),
                min = 110, max = 180, value = 140
            ),
            sliderInput("dpDoente", label = expression("Desvio-padrão da população Doente"),
                min = 6, max = 50, value = 23
            ),
            textInput("variavel", label = expression("Variável"),
                      value = "Glicemia (mg/dl)"),
            
            sliderInput("intervals", expression("Número de intervalos:"),
                min = 2, max = 20, value = 4
            )
        ),
        
        mainPanel(fluidRow(
            column(8, plotOutput("glicemia")),
            column(8, align="center", tableOutput("likelihood_table"))
        ))
    ))


shinyApp(ui = ui, server = server)