#
# aplicação que mostra o teorema do limite central para algumas distribuições 
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

server <- function(input, output) {
  
  # tipo de distribuição    
  distribution <- reactive ({
    input$var
  })
  
  # Média da distribuição normal   
  meanNormal <- reactive ({
      input$mean
  })

  # Desvio padrão da distribuição normal   
  sdNormal <- reactive ({
      input$stdev
  })

  # número de experimentos da binomial
  size <- reactive ({
      input$n
  })

  # proporção da binomial
  prob <- reactive ({
      input$p
  })

  # limite inferior da distribuição uniforme  
  minUniform <- reactive ({
      input$min
  })

  # limite superior da distribuição uniforme  
  maxUniform <- reactive ({
      input$max
  })

  # parâmetro r da distribuição gama 
  r <- reactive ({
      input$r
  })

  # parâmetro alfa da distribuição gama 
  alfa <- reactive ({
      input$alfa
  })
  
  # número de amostras
  sampleCount <- reactive ({
    input$sampleCount
  })
  
  # Tamanho de cada amostra da normal
  sampleSizeNormal <- reactive ({
    input$sampleSizeNormal
  })

  # Tamanho de cada amostra da distribuição gama
  sampleSizeGama <- reactive ({
      input$sampleSizeGama
  })

  # Tamanho de cada amostra da distribuição uniforme
  sampleSizeUnif <- reactive ({
      input$sampleSizeUnif
  })

  # número de observações da distribuição selecionada
  npoints <- reactive({
      switch(
          distribution(),
          "Normal" = {
              sampleSizeNormal() * sampleCount()
          },
          "Gama" = {
              sampleSizeGama() * sampleCount()
          },
          "Uniforme" = {
              sampleSizeUnif() * sampleCount()
          }
      )
    
  })

  # número de classes no histograma 
  classes <- reactive({
    input$breaks
  })
  
  # Distribuição normal
  yNormal <- reactive ({
    rnorm(npoints(), mean=meanNormal(), sd=sdNormal())
  })

  # Distribuição Uniforme
  yUniform <- reactive ({
    runif(npoints(), minUniform(), maxUniform())
  })

  # Distribuição Gama
  yGamma <- reactive ({
    rgamma(npoints(), r(), alfa())
  })
  
  # valores da Binomial
  binomial.x <- reactive ({
    x <- seq(0,size(), by = 1)
  })

  # probabilidades da binomial
  binomial.y <- reactive ({
    y <- dbinom(binomial.x(), size(), prob())
  })
  
  # traça o gráfico da distribuição selecionada
  output$distribution <- renderPlot({
      switch(
          distribution(),
          "Normal" = {
              inf_limit <- meanNormal() - 4 * sdNormal()
              sup_limit <- meanNormal() + 4 * sdNormal()
              curve(dnorm(x, meanNormal(), sdNormal()), xlim = c(inf_limit, sup_limit),
                  main = paste("N(", sprintf("%.1f", meanNormal()), ", ", sprintf("%.1f", sdNormal() ^ 2), ")", sep = ""),
                  xlab = "", ylab = "")
          },
          "Binomial" = {
              plotDistr(binomial.x(), binomial.y(), xlab = "", ylab = "",
                  main = paste(expression("Distribuição Binomial ("), sprintf("%.1f", prob()), ", ", sprintf("%.0f", size()), ")", sep = ""),
                  discrete = TRUE
              )
          },
          "Uniforme" = {
              curve(dunif(x, minUniform(), maxUniform()),
                  xlim = c(minUniform() - 0.5, maxUniform() + 0.5),
                  main = paste(expression("Distribuição Uniforme ("), sprintf("%.1f", minUniform()), ", ",
                      sprintf("%.1f", maxUniform()), ")", sep = ""), xlab = "", ylab = "")
          },
          "Gama" = {
              curve(dgamma(x, r(), alfa()), xlim = c(-0.5, r() * 3),
                  main = paste(expression("Distribuição Gamma ("), sprintf("%.1f", alfa()), ", ", sprintf("%.1f", r()), ")", sep = ""),
                  xlab = "", ylab = ""
              )
          }
      )
  })
  
  # plota o histograma da média amostra da distribuição selecionada
  output$sampleMeanDensityHistogram <- renderPlot({
      # y é uma lista com três elementos:  
      #   uma matriz de duas dimensões (tamanho da amostral, número de amostras)
      #   valor mínimo do eixo x
      #   valor máximo do eixo x
      y <- switch(distribution(), 
           "Normal" = {
             x <- matrix(yNormal(), nc = sampleCount())
             min_xaxis <- meanNormal() - 4 * sdNormal()
             max_xaxis <- meanNormal() + 4 * sdNormal()
             list(x, min_xaxis, max_xaxis)
           },
           "Uniforme" = {
               x <- matrix(yUniform(), nc = sampleCount())
               mean <- (minUniform() + maxUniform())/2
               stdev <- (maxUniform()-minUniform())/sqrt(12)
               min_xaxis <- mean - 4 * stdev
               max_xaxis <- mean + 4 * stdev
               list(x, min_xaxis, max_xaxis)
           },
           "Gama" = {
              x <- matrix(yGamma(), nc = sampleCount())
              mean <- r()/alfa()
              stdev <- sqrt(r())/alfa()
              min_xaxis <- 0
              max_xaxis <- mean + 4 * stdev
              list(x, min_xaxis, max_xaxis)
           },
           "Binomial" = {
             x<- matrix(rbinom(sampleCount(), size(), prob())/size(), nc = sampleCount())
             mean <- prob()
             stdev <- sqrt(prob()*(1-prob())/size())
             min_xaxis <- 0
             max_xaxis <- 1
             list(x, min_xaxis, max_xaxis)
           })
    # ybar é um vetor que contém a média de cada amostra da matriz y
    ybar <- apply(y[[1]], 2, mean)
    # sampleMean é a média das médias amostrais
    sampleMean <- mean(ybar)
    # variance é a variância das médias amostrais
    variance <- var(ybar)
    min_xaxis <- y[[2]]
    max_xaxis <- y[[3]]
    hist(ybar, freq=FALSE, breaks = classes(), xlab = "", ylab=expression("Densidade de Frequência"), main ="", xlim=c(min_xaxis, max_xaxis))
    limits <- par("usr")
    ypos <- par("usr")[4]/2 # obtém a metade do valor máximo do eixo y
    xpos <- min_xaxis + 3*(max_xaxis - min_xaxis)/4
    par(cex=1.0)
    text(xpos, ypos, paste(expression("Média = "), sprintf("%.2f", sampleMean)), pos = 4)
    text(xpos, 3*ypos/4, paste(expression("Variância = "), sprintf("%.4f", variance)), pos = 4)

  })
}

ui <- fluidPage(
  
  titlePanel("Teorema do Limite Central"),
  
  sidebarLayout(
    sidebarPanel(
      selectInput("var", 
                  label = expression("Distribuição"),
                  choices = c("Normal", "Binomial", "Uniforme", "Gama"),
                  selected = "Normal"),
      
      conditionalPanel(
          condition = "input.var == 'Normal'",
          sliderInput("mean", 
                      expression("Média"), 
                      min = 0, max = 100,
                      value = 10
          ),
          sliderInput("stdev", 
                      expression("Desvio padrão"), 
                      min = 0, max = 50,
                      value = 1
          ),
          sliderInput("sampleSizeNormal", 
                      "Tamanho amostral", 
                      min = 1, max = 200,
                      value = 1
          )
      ),
      
      conditionalPanel(
          condition = "input.var == 'Binomial'",
          sliderInput("n", 
                      expression("Número de experimentos"), 
                      min = 1, max = 200, step = 1,
                      value = 1
          ),
          sliderInput("p", 
                      "Probabilidade do evento", 
                      min = 0, max = 1,
                      value = 0.5
          )
      ),
      
      conditionalPanel(
          condition = "input.var == 'Uniforme'",
          sliderInput("min", 
                      expression("Valor mínimo"), 
                      min = 0, max = 10,
                      value = 5
          ),
          sliderInput("max", 
                      expression("Valor máximo"), 
                      min = 11, max = 40,
                      value = 15
          ),
          sliderInput("sampleSizeUnif", 
                      "Tamanho amostral", 
                      min = 1, max = 200,
                      value = 1
          )
      ),

      conditionalPanel(
          condition = "input.var == 'Gama'",
          sliderInput("alfa", 
                      "Alfa", 
                      min = 0.1, max = 10,
                      value = 1
          ),
          sliderInput("r", 
                      "r", 
                      min = 0.1, max = 10,
                      value = 2
          ),
          sliderInput("sampleSizeGama", 
                      "Tamanho amostral", 
                      min = 1, max = 200,
                      value = 1
          )
      ),

      sliderInput("sampleCount", 
                  expression("Número de amostras"), 
                  min = 1000, max = 100000,
                  value = 10000
      ),
      sliderInput("breaks", 
                  expression("Número de classes no histograma:"), 
                  min = 2, max = 100,
                  value = 20
      ),
      submitButton("Submeter")
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("distribution")),
        column(8, plotOutput("sampleMeanDensityHistogram"))
      )
    )
  )
)

shinyApp(ui = ui, server = server)